SYMBOLS["dorado.util.AjaxEngine"] = {
	"name" : "AjaxEngine",
	"type" : "class",
	"desc" : null,
	"alias" : "dorado.util.AjaxEngine",
	"author" : "Benny Bao (mailto:benny.bao@bstek.com)",
	"see" : [ "$ajax" ],
	"classDesc" : "Ajax引擎的客户端。\r<p>\rdorado中的Ajax引擎具备如下两个与众不同的特性：\r<ul>\r<li> <b>支持XMLHttpRequest对象池</b> -\rAjaxEngine引擎并不会为每一次的请求创建全新的XMLHttpRequest对象，而是会为XMLHttpRequest建立一个对象池。\r每当AjaxEngine将要发出一个Ajax请求(包括利用AjaxEngine发出的同步请求)时，AjaxEngine会首先尝试到对象池租借一个空闲的XMLHttpRequest，\r只有当无法得到一个空闲XMLHttpRequest时，dorado才会创建一个新的XMLHttpRequest。 <br>\r同样，当一次请求的处理完成后，dorado也不会立刻销毁相应的XMLHttpRequest，而将他放入到对象池中备用。 <br>\r通过上述实现方式，可以有效的降低带有大量Ajax操作的页面的系统消耗，提高页面的响应速度。 </li>\r<li> <b>支持自动批量请求的功能</b> -\r远程过程访问常常被认为是Ajax应用的运行过程中较为消耗时间的一个环节，因此如果能够尽可能减少远程过程访问(即向Server发出请求)的次数，\r应当可以使页面的效率得到提高。 <br>\rAjaxEngine支持自动收集极短时间内连续向Server发出的请求，并且把它们打包成一次批量操作的请求发往服务器。 <br>\r此功能的生效须满足一些前提条件，例如这些批量请求的URL必须是一样的，不能带有Parameter，而且此功能需要服务器上的执行逻辑提供适当的支持。 </li>\r</ul>\r</p>",
	"package" : "core",
	"inheritsFrom" : [ "dorado.EventSupport" ],
	"memberOf" : "dorado.util",
	"properties" : [ {
		"name" : "EVENTS",
		"type" : "Object",
		"desc" : "用于声明该对象中所支持的所有事件。<br>\r此属性中的对象一般由dorado系统自动生成，且往往一个类型的所有实例都共享同一个EVENTS对象。\r因此，如无特殊需要，我们不应该在运行时手动的修改此对象中的内容。",
		"alias" : "dorado.EventSupport#EVENTS",
		"example" : [ {
			"title" : "example",
			"type" : "",
			"name" : "",
			"isOptional" : false,
			"defaultValue" : "",
			"desc" : "// 获取某对象的onClick事件的声明。\rvar eventDef = oop.ENENTS.onClick。"
		} ],
		"visibility" : "public",
		"memberOf" : "dorado.EventSupport",
		"package" : "core"
	} ],
	"attributes" : [
			{
				"name" : "defaultOptions",
				"type" : "Object",
				"desc" : "默认的执行选项。 执行选项中可包含下列一些属性：\r<ul>\r<li>url - {String} 请求的URL。</li>\r<li>method - {String} 请求时的HttpMethod，可选的值包括GET、POST、PUT、DELETE。默认将按照GET来处理。</li>\r<li>header - {Object} 请求时包含在HttpRequest中的头信息 所有的头信息以属性的方式存放在json对象中。\r最终在请求发出时所包含的头信息是此处的头信息和方法的options参数中的头信息的合集，如二者之间的属性定义有冲突，则以方法的options参数中头信息的为准。</li>\r<li>xmlData - {String} 请求时以POST方法发往服务器的XML信息。</li>\r<li>jsonData - {Object|Object[]} 请求时以POST方法发往服务器的JSON信息。</li>\r<li>message - {String} 当请求尚未结束时希望系统显示给用户的提示信息。此属性目前仅以异步模式执行时有效。</li>\r</ul>\r上述属性中xmlData和jsonData在定义时只可选择其一。如果同时定义了这两个属性将只有xmlData会生效。",
				"alias" : "dorado.util.AjaxEngine#attribute:defaultOptions",
				"example" : [ {
					"title" : "example",
					"type" : "",
					"name" : "",
					"isOptional" : false,
					"defaultValue" : "",
					"desc" : "ajaxEngine.set(\"defaultOptions\", {\r\t\t url: \"/xxx.do\"\r\t\t method: \"POST\",\r\t\t headers: {\r\t\t\t \"content-type\": \"test/xml\",\r\t\t\t \"sample-header\": \"xxxxx\"\r\t\t }\r\t });"
				} ],
				"visibility" : "public",
				"memberOf" : "dorado.util.AjaxEngine",
				"package" : "core"
			},
			{
				"name" : "autoBatchEnabled",
				"type" : "boolean",
				"desc" : "是否启用自动批量请求的功能。默认值为false。\r<p>\r此功能一般须配合minConnectInterval、maxBatchSize、defaultOptions等属性一同使用。\r如果autoBatchEnabled属性为true，当AjaxEngine开始侦测到某一次发往服务器的请求时，\r会暂时搁置该请求及之后发生的请求并开始计时，直到minConnectInterval属性指定时间耗尽或被搁置的请求的数量达到maxBatchSize属性设定的上限。\r然后AjaxEngine会将这些请求进行打包合并，最后一次批量请求的方式发往服务器。\r</p>",
				"alias" : "dorado.util.AjaxEngine#attribute:autoBatchEnabled",
				"see" : [ "dorado.util.AjaxEngine#setAutoBatchEnabled" ],
				"visibility" : "public",
				"memberOf" : "dorado.util.AjaxEngine",
				"package" : "core"
			},
			{
				"name" : "minConnectInterval",
				"type" : "int",
				"desc" : "最小的与服务器建立的连接的时间间隔(毫秒数)。此属性仅在autoBatchEnabled属性为true时生效。",
				"alias" : "dorado.util.AjaxEngine#attribute:minConnectInterval",
				"visibility" : "public",
				"defaultValue" : "50",
				"memberOf" : "dorado.util.AjaxEngine",
				"package" : "core"
			},
			{
				"name" : "maxBatchSize",
				"type" : "int",
				"desc" : "每一次批量请求中允许的最大子请求数量。此属性仅在autoBatchEnabled属性为true时生效。",
				"alias" : "dorado.util.AjaxEngine#attribute:maxBatchSize",
				"visibility" : "public",
				"defaultValue" : "20",
				"memberOf" : "dorado.util.AjaxEngine",
				"package" : "core"
			},
			{
				"name" : "listener",
				"type" : "Object|Object[]",
				"desc" : "用于简化为对象添加监听器操作的虚拟属性，该属性不支持读取操作。\r<p>\r此属性的具有多态性：\r<ul>\r<li>当要赋的值为单个的JSON对象时，我们可以以事件名作为JSON对象的属性名，以事件方法作为属性值。\r在此JSON对象中关联一到多个事件。</li>\r<li>当要赋的值为数组时，数组中的每一个JSON对象都用于描述一个事件监听器。\r监听器的描述对象是一个可包含下列3个子属性的JSON对象：\r<ul>\r<li>name - {String} 要监听的事件名称。</li>\r<li>listener - {Function} 事件监听方法。</li>\r<li>[options] - {Object} 监听选项。见<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.EventSupport#addListener')\">dorado.EventSupport#addListener</A>中的options参数的说明。</li>\r</ul>\r采用此种方式定义的目的一般是为了同时某一个事件关联多个监听器，或者为了指定事件监听器的选项。 </li>\r</ul>\r</p>\r<p>\r注意：此属性只在那些实现了<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.AttributeSupport')\">dorado.AttributeSupport</A>接口的子类中有效。\r</p>",
				"alias" : "dorado.EventSupport#attribute:listener",
				"example" : [
						{
							"title" : "example",
							"type" : "",
							"name" : "",
							"isOptional" : false,
							"defaultValue" : "",
							"desc" : "// sample 1\roop.set(\"listener\", {\r\t onFocus: function(button) {\r\t\t ... ...\r\t },\r\t onBlur: function(button) {\r\t\t ... ...\r\t }\r});"
						},
						{
							"title" : "example",
							"type" : "",
							"name" : "",
							"isOptional" : false,
							"defaultValue" : "",
							"desc" : "// sample 2\r// 利用数组一次性定义两个监听器\roop.set(\"listener\", [\r\t {\r\t\t name: \"onFocus\",\r\t\t fn: function(button) {\r\t\t\t ... ...\r\t\t }\r\t },\r\t {\r\t\t name: \"onBlur\",\r\t\t fn: function(button) {\r\t\t\t ... ...\r\t\t }\r\t }\r]);"
						} ],
				"visibility" : "public",
				"memberOf" : "dorado.EventSupport",
				"package" : "core",
				"writeOnly" : true
			} ],
	"methods" : [
			{
				"name" : "request",
				"type" : "method",
				"desc" : "发起一个异步的请求。",
				"alias" : "dorado.util.AjaxEngine#request",
				"example" : [
						{
							"title" : "example",
							"type" : "",
							"name" : "",
							"isOptional" : false,
							"defaultValue" : "",
							"desc" : "// 发起一个Ajax异步请求，使用Function作为回调对象。\rvar ajax = new AjaxEngine();\rajax.request({\r\turl: \"/delete-employee.do\",\r\tmethod: \"POST\",\r\tjsonData: [\"0001\", \"0002\", \"0005\"]\r\t// 定义要提交给服务器的信息。\r}, function(result) {\r\talert(result.responseText);\r});"
						},
						{
							"title" : "example",
							"type" : "",
							"name" : "",
							"isOptional" : false,
							"defaultValue" : "",
							"desc" : "<pre>\r// 发起一个Ajax异步请求，并声明一个回调对象。\rvar ajax = new AjaxEngine();\rajax.request({\r\ttimeout: parseInt(\"30000\"), // 设置Ajax操作的超时时间为30秒\r\turl: \"/delete-employee.do\",\r\tmethod: \"POST\",\r\txmlData: \"&lt;xml&gt;&lt;id&gt;0001&lt;/id&gt;&lt;id&gt;0002&lt;/id&gt;&lt;id&gt;0005&lt;/id&gt;&lt;/xml&gt;\"\r}, {\r\tsuccess: function(result) {\r\t\talert(\"操作成功：\" + result.responseText);\r\t},\r\tfailure: function(e) {\r\t\talert(\"操作失败：\" + e);\r\t}\r});\t\r</pre>"
						},
						{
							"title" : "example",
							"type" : "",
							"name" : "",
							"isOptional" : false,
							"defaultValue" : "",
							"desc" : "// 使用自动批量请求功能。\r// 以上的三个请求最终将被AjaxEngine打包成一个批量请求，并发往服务器。\rvar ajax = new AjaxEngine();\rajax.set(\"options\", {\r\t// 每一个支持批量操作的请求的url都必须是一致的。\r\turl: \"/delete-employee.do\",\r\t\r\t// 每一个支持批量操作的请求的method都必须是一致的。\r\tmethod: \"POST\"\r});\r\r// 启用自动批量请求功能。\rajax.setAutoBatchEnabled(true);\r\rajax.request({\r\tjsonData: \"0001\"\r}, function(result) {\r\talert(result.responseText);\r});\r\rajax.request({\r\tjsonData: \"0002\"\r}, function(result) {\r\talert(result.responseText);\r});\r\rajax.request({\r\tjsonData: \"0005\"\r}, function(result) {\r\talert(result.responseText);\r});"
						} ],
				"visibility" : "public",
				"params" : [
						{
							"desc" : "执行选项。\r<p>\r此参数有两种定义方式:\r<ul>\r<li>当此参数的类型是String时，系统直接将此参数作为请求的URL。</li>\r<li>当此参数的类型是JSON对象时，系统将此参数执行选项，并处理其中的子属性。</li>\r</ul>\r</p>\r<p>\r需要注意的是，下面的xmlData和jsonData子参数在定义时只可选择其一。如果同时定义了这两个属性将只有xmlData会生效。\r</p>",
							"type" : "String|Object",
							"name" : "options",
							"isOptional" : true
						},
						{
							"desc" : "请求的URL，如果此属性未指定则使用defaultOptions.url的定义。",
							"type" : "String",
							"name" : "options.url",
							"isOptional" : true
						},
						{
							"desc" : "请求时的HttpMethod，可选值包括GET、POST、PUT、DELETE。如果此属性未指定则使用defaultOptions.method的定义。",
							"type" : "String",
							"name" : "options.method",
							"isOptional" : true,
							"defaultValue" : "\"GET\""
						},
						{
							"desc" : "请求是附带的参数。\r这些参数以子属性值的方式保存在此JSON对象中。\r<b>对于POST方法的请求而言，如果在定义了parameter的同时又定义了xmlData或jsonData，那么parameter将被添加到url中以类似GET请求的方式发送。、\r真正通过POST方法发送的数据将是xmlData或jsonData。</b>",
							"type" : "String",
							"name" : "options.parameter",
							"isOptional" : true
						},
						{
							"desc" : "请求时包含在HttpRequest中的头信息。\r这些头信息以子属性值的方式保存在此JSON对象中。\r最终在请求发出时所包含的头信息是defaultOptions.header和此处header属性的合集，如二者之间的属性定义有冲突，则以此处header属性中的为准。",
							"type" : "String",
							"name" : "options.header",
							"isOptional" : true
						},
						{
							"desc" : "请求时以POST方法发往服务器的XML信息。",
							"type" : "String",
							"name" : "options.xmlData",
							"isOptional" : true
						},
						{
							"desc" : "请求时以POST方法发往服务器的JSON信息。",
							"type" : "Object",
							"name" : "options.jsonData",
							"isOptional" : true
						},
						{
							"desc" : "当请求尚未结束时希望系统显示给用户的提示信息。此属性目前仅以异步模式执行时有效。",
							"type" : "String",
							"name" : "options.message",
							"isOptional" : true
						},
						{
							"desc" : "回调对象。传入回调对象的结果参数是<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxResult')\">dorado.util.AjaxResult</A>对象。",
							"type" : "Function|dorado.Callback",
							"name" : "callback",
							"isOptional" : true
						} ],
				"exceptions" : [ "dorado.util.AjaxException", "Error" ],
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "requestAsync",
				"type" : "method",
				"desc" : "发起一个异步的请求。\r<p>\r此方法看起来与request()完成的功能是很像的，不同点在于此方法会忽略掉autoBatchEnabled的设置。\r即利用此方法发出的请求不会被自动批量请求功能搁置，而总是会立刻发往服务器。<br>\r<b>在通常情况下，我们不建议您直接使用此方法，而应该用request()方法替代。</b>\r</p>",
				"alias" : "dorado.util.AjaxEngine#requestAsync",
				"see" : [ "dorado.util.AjaxEngine#request" ],
				"visibility" : "protected",
				"params" : [ {
					"desc" : "执行选项，请参考本类中request()方法的options参数的描述。",
					"type" : "Object",
					"name" : "options",
					"isOptional" : true
				}, {
					"desc" : "回调对象。",
					"type" : "Function|dorado.Callback",
					"name" : "callback",
					"isOptional" : true
				} ],
				"exceptions" : [ "dorado.util.AjaxException", "Error" ],
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "requestSync",
				"type" : "method",
				"desc" : "发起一个同步的请求。",
				"alias" : "dorado.util.AjaxEngine#requestSync",
				"example" : [ {
					"title" : "example",
					"type" : "",
					"name" : "",
					"isOptional" : false,
					"defaultValue" : "",
					"desc" : "var ajax = new AjaxEngine();\rvar result = ajax.requestSync({\r\turl: \"/delete-employee.do\",\r\tmethod: \"POST\",\r\tjsonData: [\"0001\", \"0002\", \"0005\"]\r\t// 定义要提交给服务器的信息\r});\ralert(result.responseText);"
				} ],
				"visibility" : "public",
				"params" : [
						{
							"desc" : "执行选项，请参考本类中request()方法的options参数的描述。",
							"type" : "String|Object",
							"name" : "options",
							"isOptional" : true
						},
						{
							"desc" : "即使发生错误也返回一个包含异常信息的<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxResult')\">dorado.util.AjaxResult</A>，而不是抛出异常信息。默认值为false，即允许抛出异常。",
							"type" : "boolean",
							"name" : "alwaysReturn",
							"isOptional" : true
						} ],
				"returns" : {
					"desc" : "执行结果。",
					"type" : "dorado.util.AjaxResult"
				},
				"exceptions" : [ "dorado.util.AjaxException", "Error" ],
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "addListener",
				"type" : "method",
				"desc" : "添加一个事件监听器。",
				"alias" : "dorado.EventSupport#addListener",
				"visibility" : "public",
				"params" : [
						{
							"desc" : "事件名称。",
							"type" : "String",
							"name" : "name"
						},
						{
							"desc" : "事件监听方法。",
							"type" : "Function",
							"name" : "listener"
						},
						{
							"desc" : "监听选项。",
							"type" : "Object",
							"name" : "options",
							"isOptional" : true
						},
						{
							"desc" : "事件方法脚本的宿主，即事件脚本中this的含义。如果此参数为空则表示this为触发该事件的对象。",
							"type" : "Object",
							"name" : "options.scope",
							"isOptional" : true
						}, {
							"desc" : "该事件是否只支持执行一次，即当事件第一次触发之后时间监听器将被自动移除。",
							"type" : "boolean",
							"name" : "options.once",
							"isOptional" : true
						}, {
							"desc" : "延时多少毫秒后触发。",
							"type" : "int",
							"name" : "options.delay",
							"isOptional" : true
						} ],
				"returns" : {
					"desc" : "返回宿主对象自身。",
					"type" : "Object"
				},
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "removeListener",
				"type" : "method",
				"desc" : "移除一个事件监听器。",
				"alias" : "dorado.EventSupport#removeListener",
				"visibility" : "public",
				"params" : [ {
					"desc" : "事件名称。",
					"type" : "String",
					"name" : "name"
				}, {
					"desc" : "事件监听器。",
					"type" : "Function",
					"name" : "listener"
				} ],
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "clearListeners",
				"type" : "method",
				"desc" : "清除事件中的所有事件监听器。",
				"alias" : "dorado.EventSupport#clearListeners",
				"visibility" : "public",
				"params" : [ {
					"desc" : "事件名称。",
					"type" : "String",
					"name" : "name"
				} ],
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "disableListeners",
				"type" : "method",
				"desc" : "禁用所有事件的监听器。",
				"alias" : "dorado.EventSupport#disableListeners",
				"visibility" : "public",
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "enableListeners",
				"type" : "method",
				"desc" : "启用所有事件的监听器。",
				"alias" : "dorado.EventSupport#enableListeners",
				"visibility" : "public",
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "fireEvent",
				"type" : "method",
				"desc" : "触发一个事件。",
				"alias" : "dorado.EventSupport#fireEvent",
				"visibility" : "public",
				"params" : [ {
					"desc" : "事件名称。",
					"type" : "String",
					"name" : "name"
				}, {
					"desc" : "0到n个事件参数。",
					"type" : "Object",
					"name" : "args...",
					"isOptional" : true
				} ],
				"returns" : {
					"desc" : "返回事件队列的触发过程是否正常的执行结束。",
					"type" : "boolean"
				},
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			}, {
				"name" : "getListenerCount",
				"type" : "method",
				"desc" : "返回某事件中 已定义的事件监听器的个数。",
				"alias" : "dorado.EventSupport#getListenerCount",
				"visibility" : "public",
				"params" : [ {
					"desc" : "事件名称。",
					"type" : "String",
					"name" : "name"
				} ],
				"returns" : {
					"desc" : "已定义的事件监听器的个数。",
					"type" : "int"
				},
				"package" : "core",
				"memberOf" : "dorado.EventSupport"
			} ],
	"constructors" : [],
	"events" : [
			{
				"name" : "beforeRequest",
				"type" : "event",
				"desc" : "当AjaxEngine将要发出以个请求时触发的事件。",
				"alias" : "dorado.util.AjaxEngine#event:beforeRequest",
				"visibility" : "public",
				"params" : [
						{
							"desc" : "事件的发起者，即AjaxEngine本身。",
							"type" : "Object",
							"name" : "self"
						},
						{
							"desc" : "事件参数。",
							"type" : "Object",
							"name" : "arg"
						},
						{
							"desc" : "是否异步操作。",
							"type" : "boolean",
							"name" : "arg.async"
						},
						{
							"desc" : "执行选项，见<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxEngine#request')\">dorado.util.AjaxEngine#request</A>中的options参数。",
							"type" : "Object",
							"name" : "arg.options"
						} ],
				"returns" : {
					"desc" : "是否要继续后续事件的触发操作，不提供返回值时系统将按照返回值为true进行处理。",
					"type" : "boolean"
				},
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "onRequest",
				"type" : "event",
				"desc" : "当AjaxEngine某个请求执行结束后(包含因失败而结束的情况)触发的事件。",
				"alias" : "dorado.util.AjaxEngine#event:onRequest",
				"visibility" : "public",
				"params" : [
						{
							"desc" : "事件的发起者，即AjaxEngine本身。",
							"type" : "Object",
							"name" : "self"
						},
						{
							"desc" : "事件参数。",
							"type" : "Object",
							"name" : "arg"
						},
						{
							"desc" : "是否异步操作。",
							"type" : "boolean",
							"name" : "arg.async"
						},
						{
							"desc" : "本次请求得到的返回结果。，见<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxResult')\">dorado.util.AjaxResult</A>。",
							"type" : "Object",
							"name" : "arg.options"
						} ],
				"returns" : {
					"desc" : "result  是否要继续后续事件的触发操作，不提供返回值时系统将按照返回值为true进行处理。",
					"type" : "boolean"
				},
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "beforeConnect",
				"type" : "event",
				"desc" : "当AjaxEngine将要与服务器建立连接时触发的事件。",
				"alias" : "dorado.util.AjaxEngine#event:beforeConnect",
				"visibility" : "public",
				"params" : [
						{
							"desc" : "事件的发起者，即AjaxEngine本身。",
							"type" : "Object",
							"name" : "self"
						},
						{
							"desc" : "事件参数。",
							"type" : "Object",
							"name" : "arg"
						},
						{
							"desc" : "是否异步操作。",
							"type" : "boolean",
							"name" : "arg.async"
						},
						{
							"desc" : "执行选项，见<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxEngine#request')\">dorado.util.AjaxEngine#request</A>中的options参数。",
							"type" : "Object",
							"name" : "arg.options"
						} ],
				"returns" : {
					"desc" : "result  是否要继续后续事件的触发操作，不提供返回值时系统将按照返回值为true进行处理。",
					"type" : "boolean"
				},
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			},
			{
				"name" : "onConnect",
				"type" : "event",
				"desc" : "当AjaxEngine断开与服务器建立连接时触发的事件。",
				"alias" : "dorado.util.AjaxEngine#event:onConnect",
				"visibility" : "public",
				"params" : [
						{
							"desc" : "事件的发起者，即AjaxEngine本身。",
							"type" : "Object",
							"name" : "self"
						},
						{
							"desc" : "事件参数。",
							"type" : "Object",
							"name" : "arg"
						},
						{
							"desc" : "是否异步操作。",
							"type" : "boolean",
							"name" : "arg.async"
						},
						{
							"desc" : "本次请求得到的返回结果。，见<A class=\"symbol-link\" href=\"javascript:openSymbolDoc('dorado.util.AjaxResult')\">dorado.util.AjaxResult</A>。",
							"type" : "Object",
							"name" : "arg.options"
						} ],
				"returns" : {
					"desc" : "result  是否要继续后续事件的触发操作，不提供返回值时系统将按照返回值为true进行处理。",
					"type" : "boolean"
				},
				"package" : "core",
				"memberOf" : "dorado.util.AjaxEngine"
			} ]
}