package com.bstek.ide.stencil.model;

import java.util.Collection;

import org.apache.commons.lang.StringUtils;

import com.bstek.ide.core.stencil.base.annotations.StencilFieldAnnotation;
import com.bstek.ide.core.stencil.base.impls.StencilInfo;

public abstract class TableStencilInfo extends StencilInfo {
	@StencilFieldAnnotation(label = "POJO", desc = "当前要处理的POJO类的完全限定名(含Package)", type = "Class")
	private String domainName;
	@StencilFieldAnnotation(label = "Spring Bean ID", desc = "该bean中有加载保存数据的方法", dependence = "domainName", type = "String")
	private String prBeanId;
	@StencilFieldAnnotation(label = "DataProvider", desc = "用于加载当前对象的方法名", dependence = "domainName", type = "String")
	private String dataProviderMethod;
	@StencilFieldAnnotation(label = "DataResolver", desc = "用于保存当前对象信息的方法名", dependence = "domainName", type = "String")
	private String dataResolverMethod;
	@StencilFieldAnnotation(label = "要处理的属性", desc = "请选择要处理的属性", dependence = "domainName", type = "Collection")
	private Collection<ColumnInfo> columns;

	// @FieldAnnotation(label = "包名", desc = "生成的view文件以及与之对应的PR类文件存储的包名")
	// private String packageName;

	public TableStencilInfo(int order, String title) {
		super(order, title);
	}

	public String getDataProvider() {
		return this.prBeanId + "#" + this.dataProviderMethod;
	}

	public String getDataResolver() {
		return this.prBeanId + "#" + this.dataResolverMethod;
	}

	public ColumnInfo getColumnInfo(String name) {
		ColumnInfo result = null;
		for (ColumnInfo col : columns) {
			if (col.getName().equals(name)) {
				result = col;
				break;
			}
		}
		return result;
	}

	public String getPrBeanId() {
		return prBeanId;
	}

	public void setPrBeanId(String prBeanId) {
		this.prBeanId = prBeanId;
	}

	public String getDataProviderMethod() {
		return dataProviderMethod;
	}

	public void setDataProviderMethod(String dataProviderMethod) {
		this.dataProviderMethod = dataProviderMethod;
	}

	public String getDataResolverMethod() {
		return dataResolverMethod;
	}

	public void setDataResolverMethod(String dataResolverMethod) {
		this.dataResolverMethod = dataResolverMethod;
	}

	public String getName() {
		int pos = this.domainName.toString().lastIndexOf(".") + 1;
		return this.domainName.toString().substring(pos,
				this.domainName.toString().length());
	}

	public Collection<ColumnInfo> getColumns() {
		return columns;
	}

	public void setColumns(Collection<ColumnInfo> columns) {
		this.columns = columns;
	}

	// public String getPackageName() {
	// return packageName;
	// }

	// public void setPackageName(String packageName) {
	// this.packageName = packageName;
	// }

	public String getDomainName() {
		return domainName;
	}

	public void setDomainName(String domainName) {
		this.domainName = domainName;
		String fixName = StringUtils.substringAfterLast(domainName, ".");
		if ("".equals(fixName)) {
			fixName = domainName;
		}
		firePropertyChange("dataProviderMethod", null, "load" + fixName);
		firePropertyChange("dataResolverMethod", null, "save" + fixName);
		firePropertyChange("prBeanId", null, StringUtils.uncapitalize(fixName)
				+ "PR");
	}

	public int getOrder() {
		return order;
	}

}
