SYMBOLS["dorado.Object"]={"name":"Object","type":"object","desc":null,"alias":"dorado.Object","author":"Benny Bao (mailto:benny.bao@bstek.com)","classDesc":"用于封装一些对象操作的类。","package":"core","memberOf":"dorado","properties":[],"attributes":[],"methods":[{"name":"createNamespace","type":"method","desc":"创建一个命名空间。","alias":"dorado.Object.createNamespace","example":[{"title":"example","type":"","name":"","isOptional":false,"defaultValue":"","desc":"// 创建新的名为\"dorado.sample\"的命名空间。\rdorado.Object.createNamespace(\"dorado.sample\");\r\r// 使用新创建的命名空间。\rdorado.sample.MyClass = function() {\r};"}],"see":["$namespace"],"visibility":"public","params":[{"desc":"命名空间的名称。例如\"dorado.sample\"。","type":"String","name":"name"}],"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"createClass","type":"method","desc":"创建并返回一个新的类。","alias":"dorado.Object.createClass","example":[{"title":"example","type":"","name":"","isOptional":false,"defaultValue":"","desc":"var MyClass = dorado.Object.createClass( {\r\t$className : \"MyClass\",\r\r\t// 构造方法\r\tconstructor : function(message) {\r\t\tthis.message = message;\r\t},\r\tgetMessage : function() {\r\t\treturn this.message;\r\t}\r});\rvar foo = new Foo(\"Hello world!\");\ralert(foo.getMessage()); // should say \"Hello world!\""}],"see":["$class"],"visibility":"public","params":[{"desc":"新类的prototype。 在该参数的子属性中有下列几个需特别注意：\r<ul>\r<li>constructor - 其中新类的构造方法可利用此属性来定义。</li>\r<li>$className - 类的名称。该名称并不会对类的执行逻辑造成影响，但是定义一个有效的名称对于JavaScript的调试将有很重要的意义。 </li>\r</ul>","type":"Object","name":"p"}],"returns":{"desc":"新的类。","type":"Object"},"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"override","type":"method","desc":"将一个类或对象中的所有属性和方法改写到另一个类中。","alias":"dorado.Object.override","visibility":"public","params":[{"desc":"被改写的类。","type":"Object","name":"subClass"},{"desc":"包含要改写的属性和方法的类或对象。","type":"Object","name":"overrides"},{"desc":"是否允许覆盖subClass中已存在的同名属性或方法。默认为不允许覆盖。","type":"boolean","name":"overwrite","isOptional":true,"defaultValue":"true"}],"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"extend","type":"method","desc":"从指定的父类派生出一个新的子类。\r<p>\r通过此方法继承出的子类具有以下的扩展属性：\r<ul>\r<li>superClass - {Prototype} 第一个父类。</li>\r<li>superClasses - {Prototype[]} 所有父类的数组。</li>\r</ul>\r上述属性的具体用法请参见示例。\r</p>","alias":"dorado.Object.extend","example":[{"title":"example","type":"","name":"","isOptional":false,"defaultValue":"","desc":"// 从SuperClass派生并得到MyClass。\rvar MyClass = dorado.Object.extend(SuperClass, {\r\t$className : \"MyClass\",\r\t//constructor是一个特殊的方法用于声明子类的构造方法。\r\tconstructor : function() {\r\t\t//调用父类的构造方法。\r\t\tSubClass.superClass.constructor.call(this, arguments);\r\t\tthis.children = [];\r},\r\r// 这是一个子类自有的方法。\r\tgetChildren : function() {\r\t\treturn this.children;\r\t}\r});"}],"see":["$extend"],"visibility":"public","params":[{"desc":"父类或父类的数组。\r如果此处定义了多个父类，那么dorado将以数组中的第一个父类作为主要的父类，新类的superClass属性将指向第一个父类。\r而其他父类中属性和方法将被继承到新类中，且后面父类中的方法和属性不会覆盖前面的同名方法或属性。","type":"Prototype|Prototype[]","name":"superClass"},{"desc":"包含一些属性和方法的类或对象，这些属性和方法将被改写进新生成的子类。 在该参数的子属性中有下列几个需特别注意：\r<ul>\r<li>constructor - 其中新类的构造方法可利用此属性来定义。</li>\r<li>$className -\r类的名称。该名称并不会对类的执行逻辑造成影响，但是定义一个有效的名称对于JavaScript的调试将有很重要的意义。 </li>\r</ul>","type":"Object","name":"overrides","isOptional":true}],"returns":{"desc":"新的子类。","type":"Prototype"},"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"eachProperty","type":"method","desc":"迭代给定对象的每一个属性。","alias":"dorado.Object.eachProperty","visibility":"public","params":[{"desc":"要迭代的对象。","type":"Object","name":"object"},{"desc":"用于监听每一个属性的迭代方法。<br/>\r该方法具有下列两个参数：\r<ul>\r<li>p - {String} 当前迭代的属性名。</li>\r<li>v - {Object} 当前迭代的属性值。 </li>\r</ul>\r该方法中的this对象即为被迭代的对象。","type":"Function","name":"fn"}],"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"apply","type":"method","desc":"将源对象中所有的属性复制（覆盖）到目标对象中。","alias":"dorado.Object.apply","example":[{"title":"example","type":"","name":"","isOptional":false,"defaultValue":"","desc":"// p, v参数即当前正在处理的属性名和属性值。\rfunction listener(p, v) {\r\tif (p == \"prop2\") {\r\t\tthis[p] = v * 2; // this即apply方法的target参数对象。\r\t\treturn false; // 返回false表示通知apply方法跳过对此属性的后续处理。\r\t}\r\telse if (p == \"prop3\") {\r\t\treturn false; // 返回false表示通知apply方法跳过对此属性的后续处理。\r\t}\r}\r\rvar target = {};\rvar source = {\r\tprop1 : 100,\r\tprop2 : 200,\r\tprop3 : 300\r};\rdorado.Object.apply(target, source, listener);\r//此时，target应为 { prop1: 100, prop2: 400 }"}],"visibility":"public","params":[{"desc":"目标对象。","type":"Object","name":"target"},{"desc":"源对象。","type":"Object","name":"source"},{"desc":"选项。\r此参数具有如下两种定义方式：\r<ul>\r<li>此值的类型是逻辑值时，表示是否覆盖目标对象中原有的属性值。\r如果设置此参数为false，那么只有当目标对象原有的属性值未定义或值为undefined时，才将源对象中的属性值写入目标对象。\r如果不定义此选项则系统默认按照覆盖方式处理。</li>\r<li>此值的类型是Function时，表示用于监听每一个属性的赋值动作的拦截方法。</li>\r</ul>","type":"boolean|Function","name":"options","isOptional":true}],"returns":{"desc":"返回目标对象。","type":"Object"},"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"isInstanceOf","type":"method","desc":"判断一个对象实例是否某类或接口的实例。\r<p>\r提供此方法的原因是因为dorado的对象集成机制是支持多重继承的，\r而Javascript中原生的instanceof操作符只能支持对多重继承中第一个超类型的判断。\r因此，当我们需要判断多重继承中的后几个超类型时，必须使用此方法。<br/>\r需要注意的是instanceof操作符的运行效率远高于此方法。\r</p>","alias":"dorado.Object.isInstanceOf","visibility":"public","params":[{"desc":"要判断的对象实例。","type":"Object","name":"object"},{"desc":"类或接口。注意：此处传入的超类或接口必须是通过dorado定义的。","type":"Function","name":"type"}],"returns":{"desc":"是否是给定类或接口的实例。","type":"boolean"},"package":"core","isStatic":true,"memberOf":"dorado.Object"},{"name":"clone","type":"method","desc":"对一个对象进行浅度克隆。","alias":"dorado.Object.clone","visibility":"public","params":[{"desc":"要克隆的对象。","type":"Object","name":"object"},{"desc":"执行选项。","type":"Object","name":"options","isOptional":true},{"desc":"用于创建克隆对象的回调函数。","type":"Function","name":"options.onCreate","isOptional":true},{"desc":"用于拦截每一个属性复制的回调函数。","type":"Function","name":"options.onCopyProperty","isOptional":true}],"returns":{"desc":"新的克隆对象。","type":"Object"},"package":"core","isStatic":true,"memberOf":"dorado.Object"}],"constructors":[],"events":[]}